package es.verdnatura.presentation.view.feature.login.model

class LoginSalixVO(
    val user: String = "",
    val password: String = "",
    val token: String = "",
    val created: String = "",
    val ttl: Long = 0,
    var isError: Boolean = false,
    var errorMessage: String = ""
)

data class WorkerData(
    val numberOfWagons: Int,
    val trainFk: Int,
    val itemPackingTypeFk: String,
    val warehouseFk: Int,
    val labelerFk: Int,
    val sectorFk: Int,
    val sector: Sector?,
    val printer: Printer,
    val train: Train,
    val id: Int,
    val name: String,
    val nickname: String,
    val isVersionCritical: Boolean,
    val version: String,
    val url: String?,
    val message: String?,
    val serialNumber: String,
    val isOnReservationMode: Boolean = false
)

data class Printer(val id: Int, val name: String)

data class Train(val id: Int, val name: String)

class DataUserSalix(
    val id: Int? = null,
    val name: String = "",
    val nickname: String = "",
)

class RenewToken(
    val id: String, val ttl: Long = 0, var isError: Boolean = false, var errorMessage: String = ""
)

class SalixMessageVO(
    val workerId: Int = 0,
    val message: String = "",
    val token: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class SalixGrupo(
    val to: String = "", val message: String = ""

)

//Tarea 5870
class OperatorSalix(
    val sector: Sector? = null,
    val train: Train? = null,
    val printer: Printer? = null,
    val numberOfWagons: Int = 0,
    var itemPackingTypeFk: String? = null,
    val isOnReservationMode: Boolean = false
)

data class Sector(
    val id: Int = 0,
    val description: String = "",
    val warehouseFk: Int = 0,
    val backupPrinterFk: Int?,
    val isOnReservationMode: Boolean? = false
)

class AccessConfigSalix(
    val id: Number = 0,
    val renewPeriod: Long = 0,
    val renewInterval: Long = 0,
)

class OperatorList(
    var list: List<OperatorSalix> = listOf()
)

class AccessConfigSalixList(
    var list: List<AccessConfigSalix> = listOf()
)

class Operator(
    val sectorFk: Int?, val labelerFk: Int?

)

data class OperatorAdd(
    var workerFk: Int, var isOnReservationMode: Boolean
)

class VersionApp(
    val appname: String = "",
    val version: String = "",
    var isVersionCritical: Boolean = false,
    var url: String? = null,
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class LoginDevice(
    val vMessage: String? = "", val vIsAuthorized: String = ""

)

data class NameWorker(
    val firstName: String, val lastName: String
)